/* ====================================================================
 * FTD4Linux Software License, Version 1.0
 *
 * Copyright (c) 2000-2004 The FTD4Linux Development Team.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        FTD4Linux Development Team (http://www.ftd4linux.2y.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "FTD" and "FTD4Linux" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact alguno@ftd4linux.2y.net.
 *
 * 5. Products derived from this software may not be called "FTD4Linux",
 *    nor may "FTD4Linux" appear in their name, without prior written
 *    permission of the FTD4Linux Development Team.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE FTD4LINUX DEVELOPMENT TEAM OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "openftd.h"
#include <libxml/parser.h>
#include <libxml/tree.h>

void
ftd_con_post_from_file(const char *filename)
{
    xmlDocPtr    doc;
    xmlNodePtr   cur;
    char *inhoud = NULL;
    GError *err = NULL;
    FtdString *str;
    FtdEvent *event;
    FtdErrorCode retval;

    if (!g_file_get_contents(filename, &inhoud, NULL, &err)) {
        g_print("Inlezen van bestand mislukt:\n%s\n", err->message);

        return;
    }

    doc = xmlParseMemory(inhoud, strlen(inhoud));
    cur = xmlDocGetRootElement(doc);
    if (cur == NULL) {
        g_print("Opgegeven bestand bevat geen bruikbare gegevens\n");
        xmlFreeDoc(doc);

        return;
    }

    if (strcasecmp(cur->name, "Event")) {
        g_print("Opgegeven bestand bevat geen bruikbare gegevens\n");
        xmlFreeDoc(doc);

        return;
    }

    cur = cur->xmlChildrenNode;
    if (!cur) {
        g_print("Opgegeven bestand bevat geen bruikbare gegevens");
        xmlFreeDoc(doc);

        return;
    }

    event = ftd_events_parse_event(doc, cur);

    g_assert(event);

    event->extra_data = ftd_events_parse_event_extra(doc, cur);

    g_assert(event->extra_data);

    xmlFreeDoc(doc);

    // Controleer of er wel voldoende gegevens zijn opgegeven
    g_assert((event->status == FTDSTATUS_SPOT) || (event->status == FTDSTATUS_ANNOUNCE) || (event->status == FTDSTATUS_UPLOADING));
    g_assert(strlen(event->fileName2) > 0);
    g_assert(strlen(event->title2) > 0);
    g_assert(event->groups && ftd_list_length(event->groups) > 0);
    g_assert(event->cat >= 0 && event->cat <= 3);
    switch(event->cat) {
        case 0: // Beeld
            g_assert(event->sCat >= 0 && event->sCat <= 5);
            break;

        case 1: // Geluid
            g_assert(event->sCat >= 0 && event->sCat <= 4);
            break;

        case 2: // Spelletjes
            g_assert(event->sCat >= 0 && event->sCat <= 9);
            break;

        case 3: // Applicaties
            g_assert(event->sCat >= 0 && event->sCat <= 5);
            break;
    }

    g_assert(event->extra_data->announcement && ftd_string_length(event->extra_data->announcement) > 0);
    g_assert(event->extra_data->description && ftd_string_length(event->extra_data->description) > 0);
    g_assert(event->extra_data->fileType >= 0 && event->extra_data->fileType <= 6);
    g_assert(event->extra_data->parType >= 0 && event->extra_data->parType <= 2);

    retval = ftd_post_send(event, NULL);
    if (retval != ERROR_NONE) {
        char *errmsg = ftd_error_msg(retval, NULL);
        g_print("%s\n", errmsg);
        g_free(errmsg);
    } else {
        g_print("Toevoegen van FTD posting geslaagd\n");
    }

    ftd_event_free(event);
}
