/* ====================================================================
 * FTD4Linux Software License, Version 1.0
 *
 * Copyright (c) 2000-2004 The FTD4Linux Development Team.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        FTD4Linux Development Team (http://www.ftd4linux.2y.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "FTD" and "FTD4Linux" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact alguno@ftd4linux.2y.net.
 *
 * 5. Products derived from this software may not be called "FTD4Linux",
 *    nor may "FTD4Linux" appear in their name, without prior written
 *    permission of the FTD4Linux Development Team.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE FTD4LINUX DEVELOPMENT TEAM OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <glib.h>
#include "openftd.h"
#include "ftd_app_core.h"

/**
 * Voegt een waarde toe aan het begin van de lijst.
 *
 * @arg     list    De ::FtdList* waaraan iets toegevoegd dient te worden.
 * @arg     value   Een pointer naar hetgeen toegevoegd dient te worden.
 *
 * @return  De nieuwe ::FtdList*.
 */
FtdList *
ftd_list_prepend (FtdList *list, gpointer value)
{
    return g_slist_prepend(list, value);
}

/**
 * Voegt een waarde toe aan het einde van de lijst.
 *
 * @arg     list    De ::FtdList* waaraan iets toegevoegd dient te worden.
 * @arg     value   Een pointer naar hetgeen toegevoegd dient te worden.
 *
 * @return  De nieuwe ::FtdList*.
 */
FtdList *
ftd_list_append (FtdList *list, gpointer value)
{
    return g_slist_append(list, value);
}

/**
 * Verwijdert een waarde uit de lijst.
 *
 * @arg     list    De ::FtdList* waaruit iets verwijderd dient te worden.
 * @arg     value   Een pointer naar hetgeen verwijderd dient te worden.
 *
 * @return  De nieuwe ::FtdList*.
 */
FtdList *
ftd_list_remove (FtdList *list, gconstpointer value)
{
    return g_slist_remove(list, value);
}

/**
 * Geeft de lijst vrij.
 * Let erop dat de elementen in de lijst niet vrijgegeven worden door deze functie.
 *
 * @arg     list    De ::FtdList* die vrijgegeven dient te worden.
 */
void
ftd_list_free(FtdList *list)
{
    g_slist_free(list);
}

/**
 * Geeft de lengte van de lijst terug.
 *
 * @arg     list    De ::FtdLisr* waarvan de lengte wordt gezocht.
 *
 * @return  De lengte van de lijst.
 */
guint
ftd_list_length(FtdList *list)
{
    return g_slist_length(list);
}

/**
 * Geeft de waarde van een element uit de lijst terug.
 *
 * @arg     list    De ::FtdList* waarvan de waarde wordt gezocht.
 *
 * @return  Een pointer naar de waarde in de lijst.
 */
gpointer
ftd_list_value(FtdList *list)
{
    return list->data;
}

/**
 * Vraagt het volgende element uit de lijst terug.
 *
 * @arg     list    De ::FtdList* van het huidige element in de lijst.
 *
 * @return  De ::FtdList* van het volgende element.
 */
FtdList *
ftd_list_next(FtdList *list)
{
    return g_slist_next(list);
}

void
ftd_list_foreach (FtdList *list, GFunc func, gpointer data)
{
    g_slist_foreach(list, func, data);
}

FtdList *
ftd_list_copy(const FtdList *list)
{
    return (FtdList *) g_slist_copy((GSList *) list);
}

FtdList *
ftd_list_find(FtdList *list, gconstpointer data)
{
    return g_slist_find(list, data);
}

FtdList *
ftd_list_reverse(FtdList *list)
{
    return g_slist_reverse(list);
}

FtdList *
ftd_list_sort(FtdList *list, GCompareFunc func)
{
    return g_slist_sort((GSList *) list, func);
}
