/* ====================================================================
 * FTD4Linux Software License, Version 1.0
 *
 * Copyright (c) 2000-2004 The FTD4Linux Development Team.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        FTD4Linux Development Team (http://www.ftd4linux.2y.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "FTD" and "FTD4Linux" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact alguno@ftd4linux.2y.net.
 *
 * 5. Products derived from this software may not be called "FTD4Linux",
 *    nor may "FTD4Linux" appear in their name, without prior written
 *    permission of the FTD4Linux Development Team.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE FTD4LINUX DEVELOPMENT TEAM OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <stdio.h>
#include "openftd.h"

/**
 * Maakt een nieuwe parameter aan.
 *
 * @arg     name        De naam van de parameter.
 * @arg     value       De waarde van de parameter.
 *
 * @return  De nieuwe parameter met de gegeven inhoud.
 */
FtdHttpParameter *
ftd_http_parameter_new(const char *name, const char *value)
{
    FtdHttpParameter *parameter;

    g_assert(name);
    g_assert(value);

    parameter             = g_malloc0(sizeof(FtdHttpParameter));
    parameter->paramName  = ftd_string_new(name);
    parameter->paramValue = ftd_string_new(value);

    return parameter;
}

/**
 * Geeft een parameter vrij.
 *
 * @arg     parameter       De vrij te geven parameter.
 */
void
ftd_http_parameter_free(FtdHttpParameter *parameter)
{
    g_assert(parameter);

    ftd_string_free(parameter->paramName);
    ftd_string_free(parameter->paramValue);
    g_free(parameter);
}

/**
 * Geeft een lijst met parameters vrij.
 * De lijst zelf wordt ook vrijgegeven.
 *
 * @arg     paramList       Een lijst met ::FtdHttpParameter pointers die vrijgegeven moet worden.
 */
void
ftd_http_parameter_free_all(FtdList *paramlist)
{
    FtdList *list;

    list = paramlist;
    while (list) {
        FtdHttpParameter *param = ftd_list_value(list);
        g_assert(param);
        ftd_http_parameter_free(param);
        list = ftd_list_next(list);
    }

    ftd_list_free(paramlist);
}
