/* ====================================================================
 * FTD4Linux Software License, Version 1.0
 *
 * Copyright (c) 2000-2004 The FTD4Linux Development Team.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        FTD4Linux Development Team (http://www.ftd4linux.2y.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "FTD" and "FTD4Linux" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact alguno@ftd4linux.2y.net.
 *
 * 5. Products derived from this software may not be called "FTD4Linux",
 *    nor may "FTD4Linux" appear in their name, without prior written
 *    permission of the FTD4Linux Development Team.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE FTD4LINUX DEVELOPMENT TEAM OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <gtk/gtk.h>
#include <webkit/webkit.h>
#include <JavaScriptCore/JSStringRef.h>

#include "openftd.h"
#include "ftd_gui_browser.h"

static void load_finished(WebKitWebView* page, WebKitWebFrame* frame, gpointer data);
static WebKitNavigationResponse navigation_requested (WebKitWebView* web_view, WebKitWebFrame* frame, WebKitNetworkRequest* request, gpointer data);

const char *
ftd_gui_browser_get_name(void)
{
    return "WebKit";
}

static void
copy_func(GtkWidget *caller, gpointer data)
{
    FtdGuiBrowserData *browser_data = (FtdGuiBrowserData *) data;

    webkit_web_view_copy_clipboard(WEBKIT_WEB_VIEW(browser_data->private_widget));
}

static gboolean
button_release_func (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
    if (event->button == 3) {
        // Laad een contextmenu zien
        GtkWidget *menu = gtk_menu_new();
        GtkWidget *item;

        item = gtk_menu_item_new_with_label("Kopieren selectie");
        g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(copy_func), user_data);
        gtk_widget_show(item);
        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);

        gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, gtk_get_current_event_time());

        return TRUE;
    }

    return FALSE;
}

FtdGuiBrowserData *
ftd_gui_browser_new(GtkWidget *parent, gboolean is_paned)
{
    FtdGuiBrowserData *browser_data = (FtdGuiBrowserData *) g_malloc0(sizeof(FtdGuiBrowserData));

    browser_data->public_widget = gtk_scrolled_window_new (NULL, NULL);

    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (browser_data->public_widget),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

    browser_data->private_widget = webkit_web_view_new();
    browser_data->private_data = NULL;

    gtk_container_add(GTK_CONTAINER(browser_data->public_widget), browser_data->private_widget);

    if (is_paned) {
        gtk_paned_pack2 (GTK_PANED (parent), browser_data->public_widget, TRUE, FALSE);
    } else {
        gtk_container_add (GTK_CONTAINER (parent), browser_data->public_widget);
    }

#if 0
    if (ftd_config_get()->disableAnimatedImages) {
        gtk_html_set_animate(GTK_HTML(browser_data->private_widget), FALSE);
    }
#endif

    webkit_web_view_set_maintains_back_forward_list(WEBKIT_WEB_VIEW(browser_data->private_widget), FALSE);

    gtk_widget_show_all(browser_data->public_widget);

    g_signal_connect(browser_data->private_widget, "load-committed", G_CALLBACK(load_finished), browser_data);
    g_signal_connect(browser_data->private_widget, "button-release-event", G_CALLBACK(button_release_func), browser_data);
    g_signal_connect(browser_data->private_widget, "navigation_requested", G_CALLBACK(navigation_requested), browser_data);

    return browser_data;
}

void
ftd_gui_browser_navigate(FtdGuiBrowserData *browser_data, const char *url)
{
    FtdString *inhoud = ftd_file_load(url, FALSE);
    if (inhoud) {
        webkit_web_view_load_html_string(WEBKIT_WEB_VIEW(browser_data->private_widget), ftd_string_value(inhoud), NULL);
        ftd_string_free(inhoud);
    }
}

void
ftd_gui_browser_destroy(FtdGuiBrowserData *browser_data)
{
    gtk_widget_destroy(browser_data->private_widget);
    gtk_widget_destroy(browser_data->public_widget);
    g_free(browser_data);
}

void
ftd_gui_browser_set_element_attribute(FtdGuiBrowserData *browser_data, const char *elementId, const char *attribute, const char *value)
{
    WebKitWebFrame *frame = webkit_web_view_get_main_frame(WEBKIT_WEB_VIEW(browser_data->private_widget));
    JSGlobalContextRef ctx = webkit_web_frame_get_global_context(frame);
    JSStringRef ref;
    JSValueRef exception;

    char *js_code;

    js_code = g_strdup_printf(  "var obj = document.getElementById(\"%s\");\n"
                                "obj.%s = \"%s\";", elementId, attribute, value);

    ref = JSStringCreateWithUTF8CString(js_code);

    JSEvaluateScript(ctx, ref, NULL, NULL, 0, &exception);

    JSStringRelease(ref);
    g_free(js_code);
}

static void
load_finished(WebKitWebView* page, WebKitWebFrame* frame, gpointer data)
{
    FtdGuiBrowserData *browser_data = (FtdGuiBrowserData *) data;

    if (browser_data->load_finished_cb) {
        browser_data->load_finished_cb(browser_data->public_widget, browser_data->load_finished_data);
    }
}

void
ftd_gui_browser_set_load_finished_cb(FtdGuiBrowserData *browser_data, void (*cb) (GtkWidget *caller, gpointer data), gpointer data)
{
    browser_data->load_finished_cb = cb;
    browser_data->load_finished_data = data;
}

static WebKitNavigationResponse
navigation_requested (WebKitWebView* web_view, WebKitWebFrame* frame, WebKitNetworkRequest* request, gpointer data)
{
    FtdGuiBrowserData *browser_data = (FtdGuiBrowserData *) data;

    if (browser_data->link_clicked_cb) {
        gboolean ret = browser_data->link_clicked_cb(browser_data->public_widget, webkit_network_request_get_uri(request), browser_data->link_clicked_data);
        if (ret) {
            return WEBKIT_NAVIGATION_RESPONSE_IGNORE;
        } else {
            return WEBKIT_NAVIGATION_RESPONSE_ACCEPT;
        }
    }

    return WEBKIT_NAVIGATION_RESPONSE_ACCEPT;
}

void
ftd_gui_browser_set_link_clicked_cb(FtdGuiBrowserData *browser_data, gboolean (*cb) (GtkWidget *caller, const char *url, gpointer data), gpointer data)
{
    browser_data->link_clicked_cb = cb;
    browser_data->link_clicked_data = data;
}

GtkWidget *
ftd_gui_browser_get_widget(FtdGuiBrowserData *browser_data)
{
    return browser_data->public_widget;
}

