/* ====================================================================
 * FTD4Linux Software License, Version 1.0
 *
 * Copyright (c) 2000-2004 The FTD4Linux Development Team.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        FTD4Linux Development Team (http://www.ftd4linux.2y.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "FTD" and "FTD4Linux" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact alguno@ftd4linux.2y.net.
 *
 * 5. Products derived from this software may not be called "FTD4Linux",
 *    nor may "FTD4Linux" appear in their name, without prior written
 *    permission of the FTD4Linux Development Team.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE FTD4LINUX DEVELOPMENT TEAM OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "ftd_gui_main.h"
#include "ftd_gui_pixbuf.h"

static GtkStatusIcon *tray_icon = NULL;

GtkStatusIcon *
ftd_gui_tray_get_status_icon(void)
{
    return tray_icon;
}

static void
ftd_gui_tray_clicked(GtkWidget *caller, gpointer data)
{
    g_assert(gui);

    if (gtk_window_is_active(GTK_WINDOW(gui->window))) {
        gtk_window_iconify(GTK_WINDOW(gui->window));
        gtk_window_set_skip_taskbar_hint(GTK_WINDOW(gui->window), TRUE);
    } else {
        gtk_window_present(GTK_WINDOW(gui->window));
        if (ftd_config_get()->startMaximized) {
            gtk_window_maximize(GTK_WINDOW(gui->window));
        }
        gtk_window_set_skip_taskbar_hint(GTK_WINDOW(gui->window), FALSE);
    }
}

static void
ftd_gui_tray_popup (GtkStatusIcon *status_icon, guint button, guint activate_time, gpointer user_data)
{
    GtkWidget *menu;
    GtkWidget *item;

    menu = gtk_menu_new();

    item = gtk_menu_item_new_with_label("Toon OpenFTD");
    g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(ftd_gui_tray_clicked), NULL);
    gtk_widget_show(item);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);

    item = gtk_menu_item_new_with_label("Afsluiten");
    g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(ftd_gui_callback_afsluiten), NULL);
    gtk_widget_show(item);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);

    gtk_widget_show(menu);
    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, button, activate_time);
}

void
ftd_gui_tray_destroy(void)
{
    g_assert(gui);

    g_return_if_fail(tray_icon != NULL);

    g_object_unref(tray_icon);

    tray_icon = NULL;

    gui->trayIconActivated = FALSE;
}

gboolean
ftd_gui_tray_create(gpointer data)
{
    if (tray_icon) {
        g_warning("ftd_gui_tray_create(): tray_icon != NULL\n");
        ftd_gui_tray_destroy();
    }

    gui->trayIconActivated = FALSE;

    tray_icon = gtk_status_icon_new_from_pixbuf(GDK_PIXBUF(ftd_gui_main_get_pixbuf("icon")));

    g_signal_connect(G_OBJECT(tray_icon), "activate", G_CALLBACK(ftd_gui_tray_clicked), NULL);
    g_signal_connect(G_OBJECT(tray_icon), "popup-menu", G_CALLBACK(ftd_gui_tray_popup), NULL);

    gui->trayIconActivated = TRUE;

    return FALSE;
}

