/* ====================================================================
 * FTD4Linux Software License, Version 1.0
 *
 * Copyright (c) 2000-2004 The FTD4Linux Development Team.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        FTD4Linux Development Team (http://www.ftd4linux.2y.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "FTD" and "FTD4Linux" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact alguno@ftd4linux.2y.net.
 *
 * 5. Products derived from this software may not be called "FTD4Linux",
 *    nor may "FTD4Linux" appear in their name, without prior written
 *    permission of the FTD4Linux Development Team.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE FTD4LINUX DEVELOPMENT TEAM OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <ctype.h>
#include "ftd_gui_config.h"
#include "ftd_gui_compat_macros.h"

static void ftd_gui_config_behaviour_callback_ratingScale(GtkWidget *caller, gpointer data);

void
ftd_gui_config_behaviour_new(FtdGuiConfig *config)
{
    FtdGuiConfigBehaviour *behaviour;
    FtdList *plugins, *list;
    int i;
    GtkWidget *hbox;
    GtkWidget *label;
    double val;

    g_assert(config);

    behaviour = g_malloc(sizeof(FtdGuiConfigBehaviour));
    config->behaviour = behaviour;

    behaviour->widget = gtk_vbox_new (FALSE, 10);
    gtk_widget_show (behaviour->widget);
    gtk_container_set_border_width (GTK_CONTAINER (behaviour->widget), 5);

    behaviour->updatenBijOpstarten = gtk_check_button_new_with_mnemonic ("Update de lijst met FTD postings bij het opstarten");
    gtk_widget_show (behaviour->updatenBijOpstarten);
    gtk_box_pack_start (GTK_BOX (behaviour->widget), behaviour->updatenBijOpstarten, FALSE, FALSE, 0);

    behaviour->toonCategoriseerdeLijstBijKnopNieuw = gtk_check_button_new_with_mnemonic ("Toon categoriseerde lijst indien op de knop 'Nieuw' gedrukt wordt");
    gtk_widget_show (behaviour->toonCategoriseerdeLijstBijKnopNieuw);
    gtk_box_pack_start (GTK_BOX (behaviour->widget), behaviour->toonCategoriseerdeLijstBijKnopNieuw, FALSE, FALSE, 0);

    behaviour->openPostinfoOpAchtergrond = gtk_check_button_new_with_mnemonic("Open de postinformatie op de achtergrond");
    gtk_widget_show(behaviour->openPostinfoOpAchtergrond);
    gtk_box_pack_start (GTK_BOX (behaviour->widget), behaviour->openPostinfoOpAchtergrond, FALSE, FALSE, 0);

    behaviour->completeWatchlistSynchronisation = gtk_check_button_new_with_mnemonic("Voer een complete synchronisatie uit bij het downloaden van de watchlist");
    gtk_widget_show(behaviour->completeWatchlistSynchronisation);
    gtk_box_pack_start (GTK_BOX (behaviour->widget), behaviour->completeWatchlistSynchronisation, FALSE, FALSE, 0);

    hbox = gtk_hbox_new(FALSE, 5);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(behaviour->widget), hbox, FALSE, FALSE, 5);

    behaviour->lblRatingScale = gtk_label_new("Ratingschaal : ");
    gtk_widget_show(behaviour->lblRatingScale);
    gtk_box_pack_start(GTK_BOX(hbox), behaviour->lblRatingScale, FALSE, FALSE, 0);

    behaviour->ratingScale = gtk_hscrollbar_new(NULL);
    gtk_range_set_range(GTK_RANGE(behaviour->ratingScale), 5.0, 200.0);
    gtk_range_set_increments(GTK_RANGE(behaviour->ratingScale), 1.0, 10.0);
    gtk_widget_show(behaviour->ratingScale);
    gtk_box_pack_end(GTK_BOX(hbox), behaviour->ratingScale, TRUE, TRUE, 0);

    hbox = gtk_hbox_new(FALSE, 5);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(behaviour->widget), hbox, FALSE, FALSE, 0);

    label = gtk_label_new("Beperk aantal postings in het nieuw binnen overzicht tot : ");
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

    behaviour->spinLimitResults = gtk_spin_button_new_with_range(100.0, 5000.0, 100.0);
    gtk_widget_show(behaviour->spinLimitResults);
    gtk_box_pack_end(GTK_BOX(hbox), behaviour->spinLimitResults, FALSE, FALSE, 0);

    hbox = gtk_hbox_new(FALSE, 5);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(behaviour->widget), hbox, FALSE, FALSE, 0);

    label = gtk_label_new("Beperk aantal postings in de overige overzichten tot : ");
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

    behaviour->spinLimitResultsTotal = gtk_spin_button_new_with_range(100.0, 5000.0, 100.0);
    gtk_widget_show(behaviour->spinLimitResultsTotal);
    gtk_box_pack_end(GTK_BOX(hbox), behaviour->spinLimitResultsTotal, FALSE, FALSE, 0);

    hbox = gtk_hbox_new(FALSE, 5);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(behaviour->widget), hbox, FALSE, FALSE, 0);

    label = gtk_label_new("Verwijder posts na X dagen : ");
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

    behaviour->comboVerwijderPostsNaXDagen = gtk_combo_box_new_text();
    gtk_combo_box_append_text(GTK_COMBO_BOX(behaviour->comboVerwijderPostsNaXDagen), CACHE_OPTIE1);
    gtk_combo_box_append_text(GTK_COMBO_BOX(behaviour->comboVerwijderPostsNaXDagen), CACHE_OPTIE2);
    gtk_combo_box_append_text(GTK_COMBO_BOX(behaviour->comboVerwijderPostsNaXDagen), CACHE_OPTIE3);
    gtk_combo_box_append_text(GTK_COMBO_BOX(behaviour->comboVerwijderPostsNaXDagen), CACHE_OPTIE4);
    gtk_combo_box_append_text(GTK_COMBO_BOX(behaviour->comboVerwijderPostsNaXDagen), CACHE_OPTIE5);
    gtk_combo_box_append_text(GTK_COMBO_BOX(behaviour->comboVerwijderPostsNaXDagen), CACHE_OPTIE6);
    gtk_combo_box_append_text(GTK_COMBO_BOX(behaviour->comboVerwijderPostsNaXDagen), CACHE_OPTIE7);
    gtk_combo_box_append_text(GTK_COMBO_BOX(behaviour->comboVerwijderPostsNaXDagen), CACHE_OPTIE8);
    gtk_combo_box_append_text(GTK_COMBO_BOX(behaviour->comboVerwijderPostsNaXDagen), CACHE_OPTIE9);
    gtk_combo_box_append_text(GTK_COMBO_BOX(behaviour->comboVerwijderPostsNaXDagen), CACHE_OPTIE10);
    gtk_combo_box_set_active(GTK_COMBO_BOX(behaviour->comboVerwijderPostsNaXDagen), ftd_config_get()->verwijderPostsNaXDagen);

    gtk_widget_show(behaviour->comboVerwijderPostsNaXDagen);
    gtk_box_pack_end(GTK_BOX(hbox), behaviour->comboVerwijderPostsNaXDagen, FALSE, FALSE, 0);

    g_signal_connect (G_OBJECT (behaviour->ratingScale), "value-changed", G_CALLBACK(ftd_gui_config_behaviour_callback_ratingScale), config);

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config->behaviour->updatenBijOpstarten), ftd_config_get()->updatenBijOpstarten);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config->behaviour->toonCategoriseerdeLijstBijKnopNieuw), ftd_config_get()->toonCategoriseerdeLijstBijKnopNieuw);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config->behaviour->openPostinfoOpAchtergrond), ftd_config_get()->openPostinfoOpAchtergrond);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config->behaviour->completeWatchlistSynchronisation), ftd_config_get()->completeWatchlistSynchronisation);

    val = ftd_config_get()->ratingScale;
    gtk_range_set_value(GTK_RANGE(behaviour->ratingScale), val);

    val = ftd_config_get()->limitResults;
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(config->behaviour->spinLimitResults), val);

    val = ftd_config_get()->limitResultsTotal;
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(config->behaviour->spinLimitResultsTotal), val);
}

void
ftd_gui_config_behaviour_free(FtdGuiConfig *config)
{
    g_assert(config);

    g_free(config->behaviour);
    config->behaviour = NULL;
}

static void
ftd_gui_config_behaviour_callback_ratingScale(GtkWidget *caller, gpointer data)
{
    FtdGuiConfig *config = (FtdGuiConfig *) data;
    char *txt;

    g_assert(config);

    txt = g_strdup_printf("Ratingschaal : (%.f)", gtk_range_get_value(GTK_RANGE(caller)));
    gtk_label_set_text(GTK_LABEL(config->behaviour->lblRatingScale), txt);
    g_free(txt);
}
