/* ====================================================================
 * FTD4Linux Software License, Version 1.0
 *
 * Copyright (c) 2000-2004 The FTD4Linux Development Team.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        FTD4Linux Development Team (http://www.ftd4linux.2y.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "FTD" and "FTD4Linux" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact alguno@ftd4linux.2y.net.
 *
 * 5. Products derived from this software may not be called "FTD4Linux",
 *    nor may "FTD4Linux" appear in their name, without prior written
 *    permission of the FTD4Linux Development Team.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE FTD4LINUX DEVELOPMENT TEAM OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <ctype.h>
#include "ftd_gui_config.h"
#include "ftd_gui_compat_macros.h"

static void     ftd_gui_config_display_groupFilter(GtkWidget *, gpointer);
static void     ftd_gui_config_display_htmlRenderer(GtkWidget *, gpointer);

void
ftd_gui_config_display_new(FtdGuiConfig *config, gboolean show_html_renderers)
{
    FtdGuiConfigDisplay *display;
    FtdList *plugins, *list;
    int i;
    GtkWidget *hbox;
    GtkWidget *label;

    g_assert(config);

    display = g_malloc(sizeof(FtdGuiConfigDisplay));
    config->display = display;

    display->widget = gtk_vbox_new (FALSE, 10);
    gtk_widget_show (display->widget);
    gtk_container_set_border_width (GTK_CONTAINER (display->widget), 5);

    display->toonInfoBijEenZoekresultaat = gtk_check_button_new_with_mnemonic ("Toon direct FTD posting info indien bij een\nzoekactie maar 1 resultaat is");
    gtk_widget_show (display->toonInfoBijEenZoekresultaat);
    gtk_box_pack_start (GTK_BOX (display->widget), display->toonInfoBijEenZoekresultaat, FALSE, FALSE, 0);

    display->startMaximized = gtk_check_button_new_with_mnemonic ("Start OpenFTD gemaximaliseerd");
    gtk_widget_show (display->startMaximized);
    gtk_box_pack_start (GTK_BOX (display->widget), display->startMaximized, FALSE, FALSE, 0);

    display->toonStatusBijLaden = gtk_check_button_new_with_mnemonic ("Toon status venster tijdens laden van HTML pagina");
    gtk_widget_show (display->toonStatusBijLaden);
    gtk_box_pack_start (GTK_BOX (display->widget), display->toonStatusBijLaden, FALSE, FALSE, 0);

#ifndef DARWIN
    display->trayIcon = gtk_check_button_new_with_mnemonic("Activeer tray icon functionaliteit");
    gtk_widget_show(display->trayIcon);
    gtk_box_pack_start (GTK_BOX (display->widget), display->trayIcon, FALSE, FALSE, 0);
#endif

    display->toonGUIGroupFilter = gtk_check_button_new_with_mnemonic("Toon nieuwsgroepenfilter in hoofdvenster");
    gtk_widget_show(display->toonGUIGroupFilter);
    gtk_box_pack_start (GTK_BOX (display->widget), display->toonGUIGroupFilter, FALSE, FALSE, 0);

    display->groupFilter = gtk_check_button_new_with_mnemonic("Activeer groepsfilter functionaliteit");
    gtk_widget_show(display->groupFilter);
    gtk_box_pack_start (GTK_BOX (display->widget), display->groupFilter, FALSE, FALSE, 0);

    display->geenTabbedBrowsing = gtk_check_button_new_with_mnemonic("Schakel tabbed browsing uit voor de postinfo");
    gtk_widget_show(display->geenTabbedBrowsing);
    gtk_box_pack_start (GTK_BOX (display->widget), display->geenTabbedBrowsing, FALSE, FALSE, 0);

    display->toonWatcherNotifications = gtk_check_button_new_with_mnemonic("Toon OpenFTD Watcher notifications");
    gtk_widget_show(display->toonWatcherNotifications);
    gtk_box_pack_start (GTK_BOX (display->widget), display->toonWatcherNotifications, FALSE, FALSE, 0);
#ifndef HAVE_LIBNOTIFY
    gtk_widget_set_sensitive(display->toonWatcherNotifications, FALSE);
#endif

    display->disableAnimatedImages = gtk_check_button_new_with_mnemonic("Schakel geanimeerde afbeeldingen uit");
    gtk_widget_show(display->disableAnimatedImages);
    gtk_box_pack_start (GTK_BOX (display->widget), display->disableAnimatedImages, FALSE, FALSE, 0);

    if (show_html_renderers) {
        display->gebruikFontFamily = gtk_check_button_new_with_mnemonic("Gebruik oude lettertype-instellingen (herstart vereist)");
        gtk_widget_show(display->gebruikFontFamily);
        gtk_box_pack_start (GTK_BOX (display->widget), display->gebruikFontFamily, FALSE, FALSE, 0);
        gtk_widget_set_sensitive(display->gebruikFontFamily, FALSE);       // Alleen bij de mozilla backend mag deze optie aan staan

        hbox = gtk_hbox_new(TRUE, 0);
        gtk_widget_show(hbox);
        gtk_box_pack_start (GTK_BOX (display->widget), hbox, FALSE, FALSE, 0);

        label = gtk_label_new("HTML Renderer (herstart vereist) :");
        gtk_widget_show(label);
        gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

        display->htmlRenderer = gtk_combo_box_new_text();
        gtk_widget_show(display->htmlRenderer);
        gtk_box_pack_start (GTK_BOX (hbox), display->htmlRenderer, TRUE, TRUE, 5);

        plugins = ftd_gui_config_get_html_renderers();
        list = plugins;
        if (plugins == NULL) {
            gtk_combo_box_append_text(GTK_COMBO_BOX(display->htmlRenderer), "Geen bruikbare HTML renderers gevonden!");
            gtk_combo_box_set_active(GTK_COMBO_BOX(display->htmlRenderer), 0);
        }

        i = 0;
        while (list) {
            FtdGuiConfigHTMLRenderer *item = ftd_list_value(list);

            gtk_combo_box_append_text(GTK_COMBO_BOX(display->htmlRenderer), item->name);

            if (i == 0) {
                gtk_combo_box_set_active(GTK_COMBO_BOX(display->htmlRenderer), i);
            }

            if (strcmp(ftd_string_value(ftd_config_get()->htmlRenderer), item->name) == 0) {
                gtk_combo_box_set_active(GTK_COMBO_BOX(display->htmlRenderer), i);

                if (strcmp(item->name, "Mozilla") == 0) {
                    gtk_widget_set_sensitive(display->gebruikFontFamily, TRUE);       // Alleen bij de mozilla backend mag deze optie aan staan
                }
            }

            i++;
            list = ftd_list_next(list);
        }

        ftd_gui_config_free_html_renderers(plugins);
    }

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config->display->toonInfoBijEenZoekresultaat), ftd_config_get()->toonInfoBijEenZoekresultaat);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config->display->startMaximized), ftd_config_get()->startMaximized);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config->display->toonStatusBijLaden), ftd_config_get()->toonStatusBijLaden);
#ifndef DARWIN
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config->display->trayIcon), ftd_config_get()->trayIcon);
#endif
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config->display->toonGUIGroupFilter), ftd_config_get()->toonGUIGroupFilter);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config->display->groupFilter), ftd_config_get()->groupFilter);
    if (show_html_renderers) {
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config->display->gebruikFontFamily), ftd_config_get()->gebruikFontFamily);
    }
#ifdef HAVE_LIBNOTIFY
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config->display->toonWatcherNotifications), ftd_config_get()->toonWatcherNotifications);
#endif
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config->display->geenTabbedBrowsing), ftd_config_get()->geenTabbedBrowsing);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config->display->disableAnimatedImages), ftd_config_get()->disableAnimatedImages);

    g_signal_connect((gpointer)config->display->groupFilter, "toggled",
                     G_CALLBACK(ftd_gui_config_display_groupFilter), config);

    if (show_html_renderers) {
        g_signal_connect((gpointer)config->display->htmlRenderer, "changed",
                         G_CALLBACK(ftd_gui_config_display_htmlRenderer), config);
    }
}

void
ftd_gui_config_display_free(FtdGuiConfig *config)
{
    g_assert(config);

    g_free(config->display);
    config->display = NULL;
}

static void
ftd_gui_config_display_groupFilter(GtkWidget *caller, gpointer data)
{
    FtdGuiConfig *config = (FtdGuiConfig *) data;

    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(config->display->groupFilter))) {
        GtkWidget *label;
        ftd_gui_config_groupfilter_new(config);
        label = gtk_label_new("Groepsfilter");
        gtk_widget_show(label);
        gtk_notebook_append_page(GTK_NOTEBOOK(config->notebook), config->groupfilter->widget, label);
    } else {
        ftd_gui_config_groupfilter_free(config);
    }
}

static void
ftd_gui_config_display_htmlRenderer(GtkWidget *caller, gpointer data)
{
    FtdGuiConfig *config = (FtdGuiConfig *) data;
    char *text = gtk_combo_box_get_active_text(GTK_COMBO_BOX(caller));

    if (strcmp(text, "Mozilla") == 0) {
        gtk_widget_set_sensitive(config->display->gebruikFontFamily, TRUE);
    } else {
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config->display->gebruikFontFamily), FALSE);
        gtk_widget_set_sensitive(config->display->gebruikFontFamily, FALSE);
    }
}
