/* ====================================================================
 * FTD4Linux Software License, Version 1.0
 *
 * Copyright (c) 2000-2004 The FTD4Linux Development Team.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        FTD4Linux Development Team (http://www.ftd4linux.2y.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "FTD" and "FTD4Linux" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact alguno@ftd4linux.2y.net.
 *
 * 5. Products derived from this software may not be called "FTD4Linux",
 *    nor may "FTD4Linux" appear in their name, without prior written
 *    permission of the FTD4Linux Development Team.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE FTD4LINUX DEVELOPMENT TEAM OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef _FTD_IRC_H_
#define _FTD_IRC_H_

typedef struct FTD_IRCObject FTD_IRCObject;

#define FTD_IRC_TYPE_OBJECT              (ftd_irc_object_get_type ())
#define FTD_IRC_OBJECT(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), FTD_IRC_TYPE_OBJECT, FTD_IRCObject))
#define FTD_IRC_OBJECT_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), FTD_IRC_TYPE_OBJECT, FTD_IRCObjectClass))
#define FTD_IRC_IS_OBJECT(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), FTD_IRC_TYPE_OBJECT))
#define FTD_IRC_IS_OBJECT_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), FTD_IRC_TYPE_OBJECT))
#define FTD_IRC_OBJECT_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), FTD_IRC_TYPE_OBJECT, FTDEventObjectClass))

typedef enum {
    IRC_STATE_DISCONNECTED,
    IRC_STATE_CONNECTING,
    IRC_STATE_CONNECTED
} FtdIRCState;

FTD_IRCObject   *ftd_irc_object_new(void);
void             ftd_irc_object_destroy(FTD_IRCObject *obj);

void             ftd_irc_object_send_msg(FTD_IRCObject *obj, const char *dest, const char *msg, gboolean is_action);
gboolean         ftd_irc_object_connect(FTD_IRCObject *obj);
void             ftd_irc_object_disconnect(FTD_IRCObject *obj);
void             ftd_irc_object_join(FTD_IRCObject *obj, const char *channel_name);
void             ftd_irc_object_change_nick(FTD_IRCObject *obj, const char *nick);


const char      *ftd_irc_object_get_active_nickname(FTD_IRCObject *obj);
void             ftd_irc_object_set_nickname(FTD_IRCObject *obj, const char *nickname);
void             ftd_irc_object_set_alt_nickname(FTD_IRCObject *obj, const char *nickname);
FtdIRCState      ftd_irc_object_get_state(FTD_IRCObject *obj);

void             ftd_irc_object_set_state_changed_cb(FTD_IRCObject *obj, void (*state_changed_cb) (FTD_IRCObject *obj, FtdIRCState old_state, const char *errmsg, gpointer data));
void             ftd_irc_object_set_msg_received_cb(FTD_IRCObject *obj, void (*on_msg_received_cb) (FTD_IRCObject *obj, const char *source, const char *channel_name, const char *msg, gboolean is_action, gboolean is_privmsg, gpointer data));
void             ftd_irc_object_set_join_cb(FTD_IRCObject *obj, void (*on_join_cb) (FTD_IRCObject *obj, const char *channel_name, const FtdList *users, const char *topic, gpointer data));
void             ftd_irc_object_set_user_join_part_cb(FTD_IRCObject *obj, void (*on_user_join_part_cb) (FTD_IRCObject *irc_data, const char *username, const char *channel, const char *reason, gboolean has_joined));
void             ftd_irc_object_set_mode_change_cb(FTD_IRCObject *obj, void (*on_mode_change_cb) (FTD_IRCObject *irc_data, const char *change_performed_by, const char *username, char *mode));
void             ftd_irc_object_set_nick_change_cb(FTD_IRCObject *obj, void (*on_nick_change_cb) (FTD_IRCObject *irc_data, const char *old_nickname, const char *new_nickname));

#endif /* _FTD_IRC_H_ */

