/* ====================================================================
 * FTD4Linux Software License, Version 1.0
 *
 * Copyright (c) 2000-2004 The FTD4Linux Development Team.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        FTD4Linux Development Team (http://www.ftd4linux.2y.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "FTD" and "FTD4Linux" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact alguno@ftd4linux.2y.net.
 *
 * 5. Products derived from this software may not be called "FTD4Linux",
 *    nor may "FTD4Linux" appear in their name, without prior written
 *    permission of the FTD4Linux Development Team.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE FTD4LINUX DEVELOPMENT TEAM OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <dbus/dbus-glib.h>

int main(int argc, char **argv)
{
    DBusGConnection *connection = NULL;
    DBusGProxy *proxy = NULL;
    GError *error;
    int postID;

    g_assert(argv[1] != NULL);

    postID = atoi(argv[1] + strlen("ftd://"));

    g_type_init ();

    error = NULL;
    connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
    if (connection == NULL)
    {
        g_print("Maken van verbinding met DBus mislukt: %s\n", error->message);
        g_error_free (error);

        return 1;
    }

    /* Create a proxy object for the "bus driver" (name "org.freedesktop.DBus") */
    proxy = dbus_g_proxy_new_for_name (connection,
                                       "org.OpenFTD.OpenFTD",
                                       "/org/OpenFTD/OpenFTD/object",
                                       "org.OpenFTD.OpenFTD");

    g_assert(proxy);

    error = NULL;
    if (!dbus_g_proxy_call (proxy, "bringToFront", &error, G_TYPE_INVALID, G_TYPE_INVALID)) {
        if (error->code == DBUS_GERROR_SERVICE_UNKNOWN) {
            // Hey, er draait nog geen instantie van FTD4Linux, opstarten maar!
            g_spawn_command_line_async("ftd", NULL);

            // Slaap nog even een paar seconden om ervoor te zorgen dat de DBUS service
            // van het FTD4Linux proces operationeel is
            sleep(5);

            g_error_free (error);
        } else {
            if (error->domain == DBUS_GERROR && error->code == DBUS_GERROR_REMOTE_EXCEPTION)
                g_printerr ("Caught remote method exception %s: %s", dbus_g_error_get_name (error), error->message);
            else
                g_printerr ("Error: %s\n", error->message);

            g_error_free (error);
            return 1;
        }
    }

    error = NULL;
    if (!dbus_g_proxy_call (proxy, "openPosting", &error, G_TYPE_INT, postID, G_TYPE_INVALID, G_TYPE_INVALID)) {
        if (error->domain == DBUS_GERROR && error->code == DBUS_GERROR_REMOTE_EXCEPTION)
            g_printerr ("Caught remote method exception %s: %s", dbus_g_error_get_name (error), error->message);
        else
            g_printerr ("Error: %s\n", error->message);

        g_error_free (error);
        return 1;
    }

    g_object_unref (proxy);

    return 0;
}

